"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.install = void 0;
const commander_1 = require("commander");
const chalk_1 = require("chalk");
const util_1 = require("util");
const lodash_1 = require("lodash");
const console_table_printer_1 = require("console-table-printer");
const index_1 = require("../../../../lib/index");
function install(cosProgram, clientProgram) {
    clientProgram
        .command("list", { isDefault: true })
        .addOption(new commander_1.Option("--detail", "查看详细信息"))
        .description("查看 COS 客户端列表")
        .action(async (opts) => {
        try {
            const { configFile } = cosProgram.opts();
            const { client } = await (0, index_1.getCOSCMDConfig)(configFile);
            const clients = client ? ((0, lodash_1.isArray)(client) ? client : [client]) : [];
            if (!clients.length) {
                console.log((0, chalk_1.red)(`缺少 COS 客户端，请先在 "cos.config.js" 文件中新增客户端`));
                return;
            }
            if (opts.detail) {
                console.log((0, util_1.inspect)(clients, { compact: false, colors: true }));
            }
            else {
                const table = new console_table_printer_1.Table({
                    title: "COS 客户端列表",
                    columns: [
                        { name: "name", alignment: "left" },
                        { name: "enable", alignment: "left" },
                        { name: "Bucket", alignment: "left" },
                        { name: "Region", alignment: "left" },
                        { name: "CDN", alignment: "left" },
                    ],
                });
                table.addRows(clients.map((client) => {
                    var _a;
                    return ({
                        name: client.name,
                        enable: !!client.enable,
                        Bucket: client.Bucket,
                        Region: client.Region,
                        CDN: (_a = client.cdn) === null || _a === void 0 ? void 0 : _a.domain,
                    });
                }), { color: "crimson" });
                table.printTable();
            }
        }
        catch (error) {
            console.log((0, chalk_1.red)("查看 COS 客户端列表失败"), error);
        }
    });
}
exports.install = install;
//# sourceMappingURL=list.js.map