import COS, { COSOptions } from "cos-nodejs-sdk-v5";
import { Client as CDNClient } from "tencentcloud-sdk-nodejs/tencentcloud/services/cdn/v20180606/cdn_client";
import { PushUrlsCacheRequest, PurgeUrlsCacheRequest } from "tencentcloud-sdk-nodejs/tencentcloud/services/cdn/v20180606/cdn_models";
import { ClientConfig as CDNClientConfig } from "tencentcloud-sdk-nodejs/tencentcloud/common/interface";
export interface BucketParams {
    Bucket: string;
    Region: string;
}
export interface SingleClientConfig extends BucketParams, COSOptions {
    enable?: boolean;
    name?: string;
    cdn?: {
        domain: string;
        config?: CDNClientConfig;
    };
}
export declare type ClientConfig = SingleClientConfig | SingleClientConfig[];
export declare type FileRes = {
    rPath: string;
    fullPath: string;
    cosKey: string;
    cosResult: any;
    cosError: null | Error;
    url: string;
};
export declare type UploadRes = {
    files: FileRes[];
    startTime: number;
    endTime: number;
};
export interface SingleUploadConfig {
    source: string;
    ignore: string[];
    cwd?: string;
    target?: string;
    rename?: boolean | number;
    flat?: boolean;
    showProgress?: boolean;
    cdnPurgeCache?: boolean | Omit<PurgeUrlsCacheRequest, "Urls">;
    cdnPushCache?: boolean | Omit<PushUrlsCacheRequest, "Urls">;
    dryRun?: boolean;
    preUpload?: (files: Pick<FileRes, "rPath" | "fullPath" | "cosKey">[]) => Promise<void>;
    postUpload?: (uploadRes: UploadRes) => Promise<void>;
}
export declare type UploadConfig = SingleUploadConfig | SingleUploadConfig[];
export interface COSCMDConfig {
    client?: ClientConfig;
    upload?: UploadConfig;
}
export declare type COSCMDConfigFileContent = COSCMDConfig | (() => Promise<COSCMDConfig>);
/**
 * 获取运行时配置，默认读取项目和电脑用户根目录下的 cos.config.js 文件，也可以指定一个配置文件
 * @param configFile 指定配置文件路径
 */
export declare function getCOSCMDConfig(configFile?: string): Promise<COSCMDConfig>;
export declare type PartialBy<T, K extends keyof T> = Omit<T, K> & Partial<Pick<T, K>>;
export declare type TupleShift<T extends any[]> = T extends [first: any, ...rest: infer R] ? R : never;
export declare type ProxiedCOS = {
    [P in Exclude<keyof COS, "uploadFiles">]: Parameters<COS[P]>[0] extends BucketParams ? (params: PartialBy<Parameters<COS[P]>[0], "Bucket" | "Region">, ...args: TupleShift<Parameters<COS[P]>>) => ReturnType<COS[P]> : COS[P];
} & {
    uploadFiles: (params: Omit<Parameters<COS["uploadFiles"]>[0], "files"> & {
        files: PartialBy<COS.UploadFileItemParams, "Bucket" | "Region">[];
    }, ...args: TupleShift<Parameters<COS["uploadFiles"]>>) => ReturnType<COS["uploadFiles"]>;
};
export declare type cosRewriteMethodNames = {
    [P in keyof COS]: Parameters<COS[P]>[0] extends BucketParams ? P : never;
}[keyof COS] | "uploadFiles";
export declare class Client {
    config: SingleClientConfig;
    static cosProxiedMethods: cosRewriteMethodNames[];
    /**
     * 生成随机字符串
     * @param {number} [length = 16] 字符串长度，默认 16
     * @returns {string} 随机字符串
     */
    static randomString(length?: number): string;
    /**
     * 查找一组字符串的最长公共前缀 https://leetcode-cn.com/problems/longest-courlPmon-prefix/
     * @param {string[]}  strs 一组字符串
     * @returns {string}  最长公共前缀
     */
    static longestCommonPrefix(strs: string[]): string;
    domain: string;
    cos: ProxiedCOS;
    cdn?: CDNClient;
    constructor(config: SingleClientConfig);
    /**
     * 上传文件
     * @param uploadConfig
     * @returns
     */
    upload(uploadConfig: SingleUploadConfig): Promise<UploadRes>;
}
//# sourceMappingURL=index.d.ts.map