"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Client = exports.getCOSCMDConfig = void 0;
const os_1 = __importDefault(require("os"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const util_1 = require("util");
const debug_1 = __importDefault(require("debug"));
const lodash_1 = require("lodash");
const is_valid_domain_1 = __importDefault(require("is-valid-domain"));
const glob_1 = __importDefault(require("glob"));
const chalk_1 = require("chalk");
const cli_progress_1 = require("cli-progress");
const cos_nodejs_sdk_v5_1 = __importDefault(require("cos-nodejs-sdk-v5"));
const cdn_client_1 = require("tencentcloud-sdk-nodejs/tencentcloud/services/cdn/v20180606/cdn_client");
// 运行时设置环境变量 DEBUG=coscmd，可以查看 debug 输出
const debug = (0, debug_1.default)("coscmd");
/**
 * 获取运行时配置，默认读取项目和电脑用户根目录下的 cos.config.js 文件，也可以指定一个配置文件
 * @param configFile 指定配置文件路径
 */
async function getCOSCMDConfig(configFile) {
    try {
        debug(`getCOSCMDConfig start`);
        const configFileNames = [".cosconfigrc", "cos.config.js", "cos.config.cjs"]; // 后面的优先级高
        const configFiles = configFile
            ? [path_1.default.resolve(configFile)]
            : [os_1.default.homedir(), process.cwd()]
                .map((base) => configFileNames.map((configFileName) => path_1.default.join(base, configFileName)))
                .flat();
        const config = await configFiles.reduce(async (promise, file) => {
            try {
                const exists = fs_1.default.existsSync(file);
                debug(`${file}${exists ? "" : " not"} exists`);
                if (!exists) {
                    return promise;
                }
                const preConfig = await promise;
                debug(`getCOSCMDConfig from ${file} start`);
                const curConfig = await Promise.resolve().then(() => __importStar(require(file))).then((v) => v.default);
                if (typeof curConfig === "function") {
                    const resolvedCurConfig = await curConfig();
                    debug(`getCOSCMDConfig from ${file} success`, resolvedCurConfig);
                    return {
                        ...preConfig,
                        ...resolvedCurConfig,
                    };
                }
                debug(`getCOSCMDConfig from ${file} success`, curConfig);
                return {
                    ...preConfig,
                    ...curConfig,
                };
            }
            catch (error) {
                debug(`getCOSCMDConfig from ${file} fail`, error);
                return promise;
            }
        }, Promise.resolve({}));
        debug(`getCOSCMDConfig success, merged config: `, config);
        return config;
    }
    catch (error) {
        debug(`getCOSCMDConfig fail`, error);
        throw error;
    }
}
exports.getCOSCMDConfig = getCOSCMDConfig;
class Client {
    constructor(config) {
        this.config = config;
        const { enable, name, SecretId, SecretKey, Bucket, Region, cdn, ...cosConfig } = config;
        // 实例化 COS
        this.cos = new Proxy(new cos_nodejs_sdk_v5_1.default({
            SecretId,
            SecretKey,
            ...cosConfig,
        }), {
            get: (target, key, recever) => {
                if (key === "uploadFiles") {
                    return function (params, ...rest) {
                        return target[key].call(target, {
                            ...params,
                            // 自动补上 Bucket 和 Region 参数，这样就不需要每次调用 cos 方法的时候都传这个参数了
                            files: params.files.map((file) => ({ Bucket, Region, ...file })),
                        }, ...rest);
                    };
                }
                else if (Client.cosProxiedMethods.includes(key)) {
                    // @ts-ignore 可耻的用一下，实在想不出来怎么处理
                    return function (params, ...rest) {
                        return target[key].call(target, 
                        // 自动补上 Bucket 和 Region 参数，这样就不需要每次调用 cos 方法的时候都传这个参数了
                        { Bucket, Region, ...params }, 
                        // @ts-ignore +1
                        ...rest);
                    };
                }
                return Reflect.get(target, key, recever);
            },
        });
        // 实例化 CDN
        if (cdn &&
            (cdn.config || (SecretId && SecretKey)) &&
            (0, is_valid_domain_1.default)(cdn.domain, { allowUnicode: true, subdomain: true, topLevel: false, wildcard: false })) {
            this.domain = cdn.domain;
            this.cdn = new cdn_client_1.Client(cdn.config || {
                credential: { secretId: SecretId, secretKey: SecretKey },
                region: "",
                profile: { httpProfile: { endpoint: "cdn.tencentcloudapi.com" } },
            });
        }
        else {
            this.domain = `${Bucket}.cos.${Region}.myqcloud.com`;
        }
    }
    /**
     * 生成随机字符串
     * @param {number} [length = 16] 字符串长度，默认 16
     * @returns {string} 随机字符串
     */
    static randomString(length = 16) {
        const alphabet = "abcdefghijklmnopqrstuvwxyz0123456789";
        const randomArray = new Array(length);
        for (let i = 0; i < length; i++) {
            randomArray[i] = alphabet.charAt(Math.floor(Math.random() * alphabet.length));
        }
        return randomArray.join("");
    }
    /**
     * 查找一组字符串的最长公共前缀 https://leetcode-cn.com/problems/longest-courlPmon-prefix/
     * @param {string[]}  strs 一组字符串
     * @returns {string}  最长公共前缀
     */
    static longestCommonPrefix(strs) {
        if (!strs.length) {
            return "";
        }
        // 最长公共前缀不可能比最短的字符串长，先遍历一次找出最短字符串
        const minStr = strs.reduce((min, str) => (min.length < str.length ? min : str), strs[0]);
        // 如果最短的那个是空字符串，直接返回 ''
        if (!minStr.length) {
            return "";
        }
        /**
         * 二分纵向比较
         * @param index 二分的位置
         * @param checkStr 待检查的字符串
         * @param commonStr 已检查过相同的部分
         * @returns
         */
        const findCommon = (index, checkStr, commonStr) => {
            const leftPart = checkStr.substr(0, index);
            let allEqual = true;
            for (let i = 0; i < strs.length; i++) {
                if (strs[i].slice(commonStr.length, commonStr.length + index) !== leftPart) {
                    allEqual = false;
                    break;
                }
            }
            if (allEqual) {
                const rightPart = checkStr.substr(index);
                commonStr += leftPart;
                // 如果纵向比较都相同且右半边没有待检查的子串，说明已经检查完毕，可以返回 commonStr 了，如果还有右半边的部分没检查，继续检查右半边的子串
                return rightPart.length ? findCommon(Math.ceil(rightPart.length / 2), rightPart, commonStr) : commonStr;
            }
            // 如果纵向比较不相同且子串的长度是1，说明这部分不是相同子串，返回已找到的 commonStr，否则继续二分
            return index === 1 ? commonStr : findCommon(Math.ceil(leftPart.length / 2), leftPart, commonStr);
        };
        return findCommon(Math.ceil(minStr.length / 2), minStr, "");
    }
    /**
     * 上传文件
     * @param uploadConfig
     * @returns
     */
    async upload(uploadConfig) {
        try {
            debug(`upload start`, uploadConfig);
            const { source, ignore, cwd = process.cwd(), target = "", rename, flat, showProgress, cdnPurgeCache, cdnPushCache, dryRun, preUpload, postUpload, } = uploadConfig;
            if (!source) {
                throw new Error("source should be nonempty string");
            }
            const normalizedTarget = target.startsWith("/") ? target.slice(1) : target;
            let files = [];
            const uploadRes = {
                files: [],
                startTime: Date.now(),
                endTime: 0,
            };
            const sourcePath = path_1.default.resolve(cwd, source);
            if (fs_1.default.existsSync(sourcePath)) {
                const stat = await fs_1.default.promises.stat(sourcePath);
                if (stat.isFile()) {
                    debug(`upload "source" is file`);
                    files = [sourcePath].map((fullPath) => {
                        const parsed = path_1.default.parse(fullPath);
                        const cosKey = parsed.ext === path_1.default.extname(normalizedTarget)
                            ? normalizedTarget
                            : path_1.default.posix.join(normalizedTarget, `${rename ? Client.randomString(rename === true ? 16 : rename) : parsed.name}${parsed.ext}`);
                        return { rPath: path_1.default.relative(cwd, fullPath), fullPath, cosKey };
                    });
                }
                else if (stat.isDirectory()) {
                    debug(`upload "source" is directory`);
                    const dirname = path_1.default.relative(cwd, sourcePath);
                    files = (await (0, util_1.promisify)(glob_1.default)(`${dirname}/**`, { cwd, nodir: true, dot: true, ignore })).map((matchedPath) => {
                        const fullPath = path_1.default.resolve(cwd, matchedPath);
                        const renamedFilePath = rename
                            ? path_1.default.format({
                                ...(0, lodash_1.pick)(path_1.default.parse(fullPath), ["dir", "ext"]),
                                name: Client.randomString(rename === true ? 16 : rename),
                            })
                            : fullPath;
                        const cosKey = path_1.default.posix.join(normalizedTarget, flat ? path_1.default.basename(renamedFilePath) : path_1.default.relative(sourcePath, renamedFilePath));
                        return { rPath: path_1.default.relative(cwd, fullPath), fullPath, cosKey };
                    });
                }
                else {
                    debug(`upload "source" not support`);
                    throw new Error("unsupport file type");
                }
            }
            else {
                debug(`use upload "source" as glob pattern`);
                const fullPaths = (await (0, util_1.promisify)(glob_1.default)(source, { cwd, nodir: true, dot: true, ignore })).map((matchedPath) => path_1.default.resolve(cwd, matchedPath));
                if (fullPaths.length) {
                    // 找出所有匹配到的文件的公共访问路径
                    const commonPrefix = Client.longestCommonPrefix(fullPaths);
                    const commonDir = commonPrefix.endsWith("/") ? commonPrefix : path_1.default.dirname(commonPrefix);
                    files = fullPaths.map((fullPath) => {
                        const renamedFilePath = rename
                            ? path_1.default.format({
                                ...(0, lodash_1.pick)(path_1.default.parse(fullPath), ["dir", "ext"]),
                                name: Client.randomString(rename === true ? 16 : rename),
                            })
                            : fullPath;
                        const cosKey = path_1.default.posix.join(normalizedTarget, flat ? path_1.default.basename(renamedFilePath) : path_1.default.relative(commonDir, renamedFilePath));
                        return { rPath: path_1.default.relative(cwd, fullPath), fullPath, cosKey };
                    });
                }
            }
            debug(`upload "source" total count is ${files.length}`);
            if (typeof preUpload === "function") {
                await preUpload(files);
            }
            if (dryRun) {
                uploadRes.files = files.map(({ rPath, fullPath, cosKey }) => ({
                    rPath,
                    fullPath,
                    cosKey,
                    cosResult: null,
                    cosError: null,
                    url: `https://${this.domain}/${cosKey}`,
                }));
            }
            else if (files.length) {
                await new Promise((resolve) => {
                    const maxPathLength = Math.max(...files.map(({ rPath }) => rPath.length));
                    const totalCount = files.length;
                    this.cos.uploadFiles({
                        files: files.map(({ rPath, fullPath, cosKey }) => ({
                            FilePath: fullPath,
                            Key: cosKey,
                            onFileFinish: (cosError, cosResult) => {
                                const url = cosError ? "" : `https://${this.domain}/${cosKey}`;
                                if (!showProgress) {
                                    console.log(`${rPath} ${(0, chalk_1.gray)(`${"-".repeat(maxPathLength - rPath.length + 3)}>`)} ${cosError ? (0, chalk_1.red)(cosKey) : chalk_1.green.underline(url)}`);
                                }
                                const fileRes = { rPath, fullPath, cosKey, cosError, cosResult, url };
                                debug("fileRes", fileRes);
                                uploadRes.files.push(fileRes);
                                if (uploadRes.files.length === totalCount) {
                                    resolve();
                                }
                            },
                        })),
                        SliceSize: 50 * 1024 * 1024,
                        ...(showProgress
                            ? (() => {
                                const bar = new cli_progress_1.SingleBar({
                                    format: `上传进度 [{bar}] {percent}% | {speed} | 文件总数 ${(0, chalk_1.cyan)("{totalCount}")} | 成功 ${(0, chalk_1.green)("{successCount}")} | 失败 ${(0, chalk_1.red)("{failCount}")}`,
                                    barsize: 30, // bar 的宽度
                                }, cli_progress_1.Presets.legacy);
                                bar.start(Number.MAX_SAFE_INTEGER, 0, {
                                    percent: 0,
                                    speed: "N/A",
                                    successCount: 0,
                                    failCount: 0,
                                    totalCount,
                                });
                                return {
                                    onProgress: ({ loaded, total, speed, percent }) => {
                                        const uploadedCount = uploadRes.files.length;
                                        const successCount = uploadRes.files.filter(({ cosError }) => !cosError).length;
                                        bar.setTotal(total);
                                        bar.update(loaded, {
                                            percent: Math.floor(percent * 100),
                                            speed: speed > 1024 * 1024
                                                ? `${(speed / 1024 / 1024).toFixed(2)}MB/s`
                                                : `${(speed / 1024).toFixed(2)}KB/s`,
                                            successCount: successCount,
                                            failCount: uploadedCount - successCount,
                                        });
                                        if (uploadedCount === totalCount) {
                                            bar.stop();
                                        }
                                    },
                                };
                            })()
                            : null),
                    });
                });
                if (this.cdn) {
                    if (cdnPurgeCache) {
                        for (let i = 0; i < uploadRes.files.length; i += 1000) {
                            const Urls = uploadRes.files.slice(i, i + 1000).map((file) => file.url);
                            // 每次最多 1000 条
                            await this.cdn.PurgeUrlsCache({ Urls, ...(cdnPurgeCache === true ? null : cdnPurgeCache) });
                        }
                        console.log((0, chalk_1.green)("CDN 缓存刷新成功"));
                    }
                    if (cdnPushCache) {
                        for (let i = 0; i < uploadRes.files.length; i += 20) {
                            const Urls = uploadRes.files.slice(i, i + 20).map((file) => file.url);
                            // 每次最多 20 条
                            await this.cdn.PushUrlsCache({ Urls, ...(cdnPushCache === true ? null : cdnPushCache) });
                        }
                        console.log((0, chalk_1.green)("CDN 缓存预热成功"));
                    }
                }
            }
            uploadRes.endTime = Date.now();
            if (typeof postUpload === "function") {
                await postUpload(uploadRes);
            }
            return uploadRes;
        }
        catch (error) {
            debug(`upload fail`, error);
            throw error;
        }
    }
}
exports.Client = Client;
Client.cosProxiedMethods = [
    "uploadFiles",
    "abortUploadTask",
    "deleteBucket",
    "deleteBucketCors",
    "deleteBucketDomain",
    "deleteBucketEncryption",
    "deleteBucketInventory",
    "deleteBucketLifecycle",
    "deleteBucketOrigin",
    "deleteBucketPolicy",
    "deleteBucketReplication",
    "deleteBucketTagging",
    "deleteBucketWebsite",
    "deleteMultipleObject",
    "deleteObject",
    "deleteObjectTagging",
    "getBucket",
    "getBucketAccelerate",
    "getBucketAcl",
    "getBucketCors",
    "getBucketDomain",
    "getBucketEncryption",
    "getBucketInventory",
    "getBucketLifecycle",
    "getBucketLocation",
    "getBucketLogging",
    "getBucketOrigin",
    "getBucketPolicy",
    "getBucketReferer",
    "getBucketReplication",
    "getBucketTagging",
    "getBucketVersioning",
    "getBucketWebsite",
    "getObject",
    "getObjectAcl",
    "getObjectStream",
    "getObjectTagging",
    "getObjectUrl",
    "headBucket",
    "headObject",
    "listBucketInventory",
    "listObjectVersions",
    "multipartAbort",
    "multipartComplete",
    "multipartInit",
    "multipartList",
    "multipartListPart",
    "multipartUpload",
    "optionsObject",
    "putBucket",
    "putBucketAccelerate",
    "putBucketAcl",
    "putBucketCors",
    "putBucketDomain",
    "putBucketEncryption",
    "putBucketInventory",
    "putBucketLifecycle",
    "putBucketLogging",
    "putBucketOrigin",
    "putBucketPolicy",
    "putBucketReferer",
    "putBucketReplication",
    "putBucketTagging",
    "putBucketVersioning",
    "putBucketWebsite",
    "putObject",
    "putObjectAcl",
    "putObjectCopy",
    "putObjectTagging",
    "request",
    "restoreObject",
    "selectObjectContent",
    "selectObjectContentStream",
    "sliceCopyFile",
    "sliceUploadFile",
    "uploadFile",
    "uploadPartCopy",
];
//# sourceMappingURL=index.js.map